@@ -16,8 +16,12 @@
 
 #define _FILE_OFFSET_BITS  64
 #include <config.h>
-#include <gnome.h>
-#include <libgnomevfs/gnome-vfs.h>
+#include <gio/gio.h> 
+#include <glib.h>
+#include <glib/gstdio.h>
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+
 #include <fcntl.h>
 #include <signal.h>
 #include <stdlib.h>
@@ -43,6 +47,8 @@ static void gwget_data_update_statistics_ui(GwgetData *gwgetdata);
 static void convert_time2str(gchar *buffer, guint32 time);
 static void gwget_gnotify_finished(GwgetData *gwgetdata);
 static void gwget_download_finished(GwgetData *gwgetdata);
+static gboolean gwget_ask_download_playlist(GwgetData *gwgetdata);
+static void gwget_download_playlist_items(gchar *filename);
 
 static void
 convert_time2str (gchar *buffer, guint32 time)
@@ -440,8 +446,8 @@ gwget_data_start_download(GwgetData *gwgetdata)
 					user_password=g_strdup_printf("%s:%s@",gwget_pref.proxy_user,gwget_pref.proxy_password);
 				}
 
-				setenv("http_proxy", g_strconcat("http://",g_strdup_printf(user_password),g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL), 1);
-				setenv("ftp_proxy", g_strconcat("http://",g_strdup_printf(user_password),g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL), 1);
+				setenv("http_proxy", g_strconcat("http://", g_strdup_printf("%s", user_password),g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL), 1);
+				setenv("ftp_proxy", g_strconcat("http://", g_strdup_printf("%s", user_password),g_strdup_printf("%s",gwget_pref.http_proxy),":",g_strdup_printf("%d",gwget_pref.http_proxy_port),NULL), 1);
 		 	        argv[arg]="-Yon";
 				arg++;
 			}
@@ -453,8 +459,8 @@ gwget_data_start_download(GwgetData *gwgetdata)
 					user_password=g_strdup_printf("%s:%s@",gwget_pref.gnome_proxy_user,gwget_pref.gnome_proxy_password);
 				}
 
-				setenv("http_proxy",g_strconcat("http://",g_strdup_printf(user_password),g_strdup_printf("%s",gwget_pref.gnome_http_proxy),":",g_strdup_printf("%d",gwget_pref.gnome_http_proxy_port),NULL),1);
-				setenv("ftp_proxy",g_strconcat("http://",g_strdup_printf(user_password),g_strdup_printf("%s",gwget_pref.gnome_http_proxy),":",g_strdup_printf("%d",gwget_pref.gnome_http_proxy_port),NULL),1);
+				setenv("http_proxy",g_strconcat("http://", g_strdup_printf("%s", user_password), g_strdup_printf("%s", gwget_pref.gnome_http_proxy),":", g_strdup_printf("%d", gwget_pref.gnome_http_proxy_port),NULL),1);
+				setenv("ftp_proxy",g_strconcat("http://", g_strdup_printf("%s", user_password), g_strdup_printf("%s", gwget_pref.gnome_http_proxy),":", g_strdup_printf("%d", gwget_pref.gnome_http_proxy_port),NULL),1);
 		 	        argv[arg]="-Yon";
 				arg++;
 			}
@@ -515,8 +521,10 @@ GwgetData *
 gwget_data_create(gchar *url, gchar *dir)
 {
 	GwgetData *gwgetdata;
-	GnomeVFSURI *localfile_uri;
+	GFile *file;
+	GFileInfo *info;
 	gint length;
+	GError    *err = NULL;
 
 	g_return_val_if_fail(url != NULL, NULL);
 	g_return_val_if_fail(dir != NULL, NULL);
@@ -544,23 +552,22 @@ gwget_data_create(gchar *url, gchar *dir)
 	
 	gwget_data_set_filename(gwgetdata,gwgetdata->filename);
 	
-	localfile_uri = gnome_vfs_uri_new (gwgetdata->local_filename);
-	if (gnome_vfs_uri_exists (localfile_uri) ) {
-		GnomeVFSFileInfo *info;
-		info = gnome_vfs_file_info_new();
-		gnome_vfs_get_file_info (gwgetdata->local_filename, 
-				         info, 
-					 GNOME_VFS_FILE_INFO_DEFAULT);
-		gwgetdata->cur_size = (guint64)info->size;
+	file = g_file_new_for_path (gwgetdata->local_filename);
+	info = g_file_query_info (file, G_FILE_ATTRIBUTE_STANDARD_SIZE, 0, NULL, &err);
+	
+	if (err==NULL) {
+		gwgetdata->cur_size = g_file_info_get_size (info);
 	} else {
 		gwgetdata->cur_size = 0;
+		g_warning ("Failed to get file size for %s: %s", gwgetdata->local_filename, err->message);
+		g_error_free (err);
 	}
 	
+	
 	gwgetdata->line = NULL;
 	gwgetdata->session_start_time = 0;
 	gwgetdata->session_start_size = 0;
 	gwgetdata->session_elapsed = 0;
-	/* gwgetdata->cur_size = 0; */
 	gwgetdata->state = DL_NOT_STARTED;
 	gwgetdata->total_size = 0;
 	gwgetdata->total_time = 0;
@@ -727,12 +734,15 @@ gwget_data_add_download(GwgetData *gwgetdata)
 void
 gwget_data_exec (GwgetData *gwgetdata)
 {
-	gchar *uri;
+	gchar *url;
+	GFile *file;
 	GError *err = NULL; 
 
-	uri = gnome_vfs_make_uri_from_input_with_dirs (gwgetdata->local_filename,
-							GNOME_VFS_MAKE_URI_DIR_CURRENT);
-	if (!gnome_url_show (uri, &err)) {
+	file = g_file_new_for_commandline_arg (gwgetdata->local_filename);
+	url = g_file_get_uri (file);
+	g_object_unref (file);
+	
+	if (!gtk_show_uri (NULL, url, GDK_CURRENT_TIME, &err)) {
 		run_dialog_error(_("Error opening file"), _("Couldn't open the file"));
 	}
 }
@@ -780,9 +790,53 @@ gwget_gnotify_finished(GwgetData *gwgetdata) {
 static void
 gwget_download_finished (GwgetData *gwgetdata)
 {
-   gwget_gnotify_finished(gwgetdata);
-   gwget_tray_notify (_("Download Complete"), gwgetdata->filename, gwgetdata->icon_name);
-   gtk_widget_set_sensitive(glade_xml_get_widget(xml, "clear_button"), TRUE);
+	gwget_gnotify_finished(gwgetdata);
+	gwget_tray_notify (_("Download Complete"), gwgetdata->filename, gwgetdata->icon_name);
+	gtk_widget_set_sensitive(glade_xml_get_widget(xml, "clear_button"), TRUE);
+
+	if (gwget_ask_download_playlist(gwgetdata))
+		gwget_download_playlist_items(gwgetdata->local_filename);
+}
+
+static gboolean
+gwget_ask_download_playlist(GwgetData *gwgetdata)
+{
+	gchar *msg;
+	gint response;
+
+	if (g_str_has_suffix(gwgetdata->filename, ".m3u")
+	    	|| g_str_has_suffix(gwgetdata->filename, ".M3U")) {   
+		msg = g_strdup_printf(
+			_("The file %s is an MP3 playlist.\nDownload the files that it contains?"),
+			gwgetdata->filename);
+		response = run_dialog(_("Download files in MP3 playlist?"),
+							  _(msg), _("No"), _("Yes"));
+		g_free(msg);
+		if (response == GTK_RESPONSE_OK)
+			return TRUE;
+	}
+	
+	return FALSE;
+}
+
+static void
+gwget_download_playlist_items(gchar *filename)
+{
+	FILE *f;
+	gchar line[1024];
+	GwgetData *gwgetdata;
+	
+	f = g_fopen(filename, "r");
+	if (f!=NULL) {
+		while (fgets(line, 1024, f)!=NULL) {
+			if (check_url("http://", line) || check_url("ftp://", line)) {
+				gwgetdata = gwget_data_new (g_strstrip(line));
+				gwget_data_add_download(gwgetdata);
+				gwget_data_start_download(gwgetdata);
+			}
+		}
+		fclose(f);
+	}
 }
 
 void
