@@ -31,6 +31,8 @@
 #include "systray.h"
 #include "new_window.h"
 
+static gboolean there_are_completed_on_startup = FALSE;
+ 
 /* Global Preferences */
 Preferences gwget_pref;
 
@@ -46,6 +48,13 @@ gwget_gconf_notify_toolbar(GConfClient *client,
 		   	   GConfEntry  *entry,
 		   	   gpointer     user_data);
 
+static gboolean
+view_selection_func (GtkTreeSelection *selection,
+                        GtkTreeModel     *model,
+                        GtkTreePath      *path,
+                        gboolean          path_currently_selected,
+                        gpointer          userdata);
+
 void 
 main_window(void) 
 {
@@ -62,19 +71,14 @@ main_window(void)
 		glade_xml_signal_autoconnect(xml);
 	}
 	
-	if (!xml_new) {
-		xml_file =g_build_filename(DATADIR, "newdownload.glade", NULL);
-		xml_new = glade_xml_new(xml_file, NULL, NULL);
-		glade_xml_signal_autoconnect (xml_new);
-	}
-	
-	
 	window = glade_xml_get_widget(xml,"main_window");
 	treev = glade_xml_get_widget(xml,"treeview1");
 	model = create_model();
 	gtk_tree_view_set_model(GTK_TREE_VIEW(treev),GTK_TREE_MODEL(model));
 	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (treev));
 	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
+	/* Set a callback when a selection is made for update the menu Download */
+	gtk_tree_selection_set_select_function(select, view_selection_func, NULL, NULL);
 
 	/*
 	 * Update window size in gconf on resize 
@@ -86,7 +90,7 @@ main_window(void)
 	
 	/* add the columns titles to the tree view */
 	add_columns (GTK_TREE_VIEW (treev));
-	
+
 	gconf_client = gconf_client_get_default();
 	gconf_client_add_dir (gconf_client, 
 			      "/apps/gwget2", 
@@ -138,7 +142,7 @@ main_window(void)
 	
 	/* Show the toolbar ? */
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget(xml,"view_toolbar")),gwget_pref.view_toolbar);
-	toolbar = glade_xml_get_widget(xml,"bonobotoolbar"); 
+	toolbar = glade_xml_get_widget(xml,"toolbar1"); 
 	menu_item=glade_xml_get_widget(GLADE_XML(xml),"view_toolbar");
 	if (gwget_pref.trayonly) {
 		gtk_widget_show(GTK_WIDGET(window));
@@ -153,18 +157,62 @@ main_window(void)
 		gtk_widget_show(GTK_WIDGET(toolbar));
 	}
 
-	
+	/* Show the statusbar ? */
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget(xml,"view_statusbar")), gwget_pref.view_statusbar);
+	menu_item=glade_xml_get_widget(GLADE_XML(xml),"view_statusbar");
+	if (gwget_pref.view_statusbar)
+	{
+		gtk_widget_show (GTK_WIDGET(glade_xml_get_widget(xml, "statusbar")));
+	} else {
+		gtk_widget_hide (GTK_WIDGET(glade_xml_get_widget(xml, "statusbar")));
+	}
+
+	if (there_are_completed_on_startup) {
+		gtk_widget_set_sensitive (glade_xml_get_widget (xml, "clear_button"), TRUE);
+	} else {
+		gtk_widget_set_sensitive (glade_xml_get_widget (xml, "clear_button"), FALSE);
+	}
+
 	systray_load();
+	gwget_pref.docked=TRUE;
 
 	/* Create the model for the "save in" option in new download dialog */
 	save_in_model = (GtkTreeModel*)gtk_list_store_new (1, G_TYPE_STRING);
-	combo = glade_xml_get_widget (xml_new, "save_in_comboboxentry");
+	combo = glade_xml_get_widget (xml, "save_in_comboboxentry");
 	gtk_combo_box_set_model(GTK_COMBO_BOX(combo), save_in_model);
 	gtk_combo_box_entry_set_text_column (GTK_COMBO_BOX_ENTRY(combo), 0);
 	gtk_entry_set_text(GTK_ENTRY(GTK_BIN(combo)->child), gwget_pref.download_dir);
 		
 }
 
+static 
+gboolean view_selection_func (GtkTreeSelection *selection,
+			GtkTreeModel     *model,
+			GtkTreePath      *path,
+			gboolean          path_currently_selected,
+			gpointer          userdata)
+{
+	GtkTreeIter iter;
+
+	if (gwget_data_get_selected() == NULL) {
+		return TRUE;
+	}
+
+	if (gtk_tree_model_get_iter(model, &iter, path))
+	{
+		gchar *name;
+
+		gtk_tree_model_get(model, &iter, FILENAME_COLUMN, &name, -1);
+
+		g_free(name);
+	}
+
+	return TRUE; /* allow selection state to change */
+}
+
+
+
+
 static gboolean
 gwget_destination_file_exists(GwgetData *data)
 {
@@ -173,10 +221,12 @@ gwget_destination_file_exists(GwgetData *data)
 	return ( g_stat(data->local_filename,&s) == 0 );
 }
 
+
 void 
 gwget_get_defaults_from_gconf(void)
 {
-	gint num_dl, i, total_size;
+	gint num_dl, i;
+	gint64 total_size;
 	GwgetData *data;
 	gchar *key,*url,*dir,*name;
 	DlState state;
@@ -185,9 +235,17 @@ gwget_get_defaults_from_gconf(void)
 	
 	gwget_pref.http_proxy=gconf_client_get_string(gconf_client,"/apps/gwget2/http_proxy",NULL);
 	gwget_pref.http_proxy_port=gconf_client_get_int(gconf_client,"/apps/gwget2/http_proxy_port",NULL);
+	gwget_pref.proxy_uses_auth=gconf_client_get_bool(gconf_client,"/apps/gwget2/proxy_uses_auth",NULL);
+	gwget_pref.proxy_user=gconf_client_get_string(gconf_client,"/apps/gwget2/proxy_user",NULL);
+	gwget_pref.proxy_password=gconf_client_get_string(gconf_client,"/apps/gwget2/proxy_password",NULL);
+	
 	gwget_pref.gnome_http_proxy=gconf_client_get_string(gconf_client,"/system/http_proxy/host",NULL);
 	gwget_pref.gnome_http_proxy_port=gconf_client_get_int(gconf_client,"/system/http_proxy/port",NULL);
+	gwget_pref.gnome_proxy_uses_auth=gconf_client_get_bool(gconf_client,"/system/http_proxy/use_authentication",NULL);
+	gwget_pref.gnome_proxy_user=gconf_client_get_string(gconf_client,"/system/http_proxy/authentication_user",NULL);
+	gwget_pref.gnome_proxy_password=gconf_client_get_string(gconf_client,"/system/http_proxy/authentication_password",NULL);
 	gwget_pref.gnome_use_proxy=gconf_client_get_bool(gconf_client,"/system/http_proxy/use_http_proxy",NULL);
+	
 	gwget_pref.network_mode=gconf_client_get_string(gconf_client,"/apps/gwget2/network_mode",NULL);
 	gwget_pref.download_dir=gconf_client_get_string(gconf_client,"/apps/gwget2/download_dir",NULL);
 	gwget_pref.num_retries=gconf_client_get_int(gconf_client,"/apps/gwget2/num_retries",NULL);
@@ -204,6 +262,7 @@ gwget_get_defaults_from_gconf(void)
 	gwget_pref.view_elapse_time=gconf_client_get_bool(gconf_client,"/apps/gwget2/view_elapse_time",NULL);
 	gwget_pref.view_down_speed=gconf_client_get_bool(gconf_client,"/apps/gwget2/view_down_speed",NULL);
 	gwget_pref.view_toolbar=gconf_client_get_bool(gconf_client,"/apps/gwget2/view_toolbar",NULL);
+	gwget_pref.view_statusbar=gconf_client_get_bool(gconf_client,"/apps/gwget2/view_statusbar",NULL);
 	gwget_pref.view_file_type=gconf_client_get_bool(gconf_client,"/apps/gwget2/view_file_type",NULL);
 	gwget_pref.limit_speed = gconf_client_get_bool (gconf_client,"/apps/gwget2/limit_speed", NULL);
 	gwget_pref.max_speed=gconf_client_get_int(gconf_client,"/apps/gwget2/max_speed",NULL);
@@ -240,8 +299,8 @@ gwget_get_defaults_from_gconf(void)
 		key=g_strdup_printf("/apps/gwget2/downloads_data/%d/state",i);
 		state=gconf_client_get_int(gconf_client,key,NULL); 
 		
-		key=g_strdup_printf("/apps/gwget2/downloads_data/%d/total_size",i);
-		total_size = gconf_client_get_int (gconf_client, key, &error);
+		key=g_strdup_printf("/apps/gwget2/downloads_data/%d/file_size",i);
+		total_size = atoll(gconf_client_get_string (gconf_client, key, &error));
 		if (!error) {		
 			gwget_data_set_total_size (data, total_size);
 		} else { 
@@ -267,20 +326,21 @@ gwget_get_defaults_from_gconf(void)
 				 */
 				continue;
 			} else {
+				there_are_completed_on_startup = TRUE;
 				new_download(data);
-				gwget_data_set_state(data,DL_COMPLETED);
+				gwget_data_set_state_no_sync(data,DL_COMPLETED);
 				continue;
 			}
 		}
 		    		
-		gwget_data_set_state(data,DL_NOT_RUNNING); 
+		gwget_data_set_state_no_sync(data,DL_NOT_RUNNING); 
 		if (gwget_pref.resume_at_start && data->state!=DL_COMPLETED) {
 			gwget_data_start_download(data);
 		} 
 		/* FIXME: put all the cases (error, retriving...) */
 		
 		if (state==DL_COMPLETED) {
-			gwget_data_set_state(data, state);
+			gwget_data_set_state_no_sync(data, state);
 		}
 	}
 	/*
@@ -345,6 +405,8 @@ add_columns (GtkTreeView *treeview)
 {
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
+	PangoContext *context;
+	PangoLayout *layout;
 	
 	
 	/* File Name Column */
@@ -369,7 +431,12 @@ add_columns (GtkTreeView *treeview)
 	
 	gtk_tree_view_column_set_title (GTK_TREE_VIEW_COLUMN(column),_("File Name"));
 	
-	gtk_tree_view_column_set_resizable(column, TRUE); 
+	context = gtk_widget_get_pango_context(GTK_WIDGET(treeview));
+	layout = pango_layout_new (PANGO_CONTEXT(context));
+	pango_layout_set_ellipsize (PANGO_LAYOUT(layout), PANGO_ELLIPSIZE_END);
+	
+	gtk_tree_view_column_set_sort_column_id (column, FILENAME_COLUMN);
+	gtk_tree_view_column_set_resizable (column, TRUE); 
 	gtk_tree_view_append_column (treeview, column);
 	
 	/* State Column */
@@ -380,6 +447,7 @@ add_columns (GtkTreeView *treeview)
 							  STATE_COLUMN,
 							  NULL);
 	gtk_tree_view_column_set_resizable(column, TRUE); 
+	gtk_tree_view_column_set_sort_column_id (column, STATE_COLUMN);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_append_column (treeview, column);
 	
@@ -392,6 +460,7 @@ add_columns (GtkTreeView *treeview)
 							NULL);
 
 	gtk_tree_view_column_set_sort_column_id (column, CURRENTSIZE_COLUMN);
+	gtk_tree_view_column_set_resizable(column, TRUE); 
   	gtk_tree_view_append_column (treeview, column);
 	
 	/* Total Size */
@@ -402,6 +471,7 @@ add_columns (GtkTreeView *treeview)
 							TOTALSIZE_COLUMN,
 							NULL);
 	gtk_tree_view_column_set_sort_column_id (column, TOTALSIZE_COLUMN);
+	gtk_tree_view_column_set_resizable(column, TRUE); 
   	gtk_tree_view_append_column (treeview, column);
 	
 	/* Percentage */
@@ -413,6 +483,8 @@ add_columns (GtkTreeView *treeview)
 							NULL);
 
 	gtk_tree_view_column_set_title (column, (_("Percentage"))); 
+	gtk_tree_view_column_set_resizable(column, TRUE); 
+	gtk_tree_view_column_set_sort_column_id (column, PERCENTAGE_COLUMN);
     	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview),column); 
 		
 	/* Elapse Time */
@@ -423,6 +495,7 @@ add_columns (GtkTreeView *treeview)
 							ELAPSETIME_COLUMN,
 							NULL);
 	gtk_tree_view_column_set_sort_column_id (column, ELAPSETIME_COLUMN);
+	gtk_tree_view_column_set_resizable(column, TRUE); 
 	gtk_tree_view_append_column (treeview, column);
 	
 	/* REMAINTIME_COLUMN */
@@ -433,6 +506,7 @@ add_columns (GtkTreeView *treeview)
 														REMAINTIME_COLUMN,
 														NULL);
 	gtk_tree_view_column_set_sort_column_id (column, REMAINTIME_COLUMN);
+	gtk_tree_view_column_set_resizable(column, TRUE); 
 	gtk_tree_view_append_column (treeview, column);
 	
 	/* Speed */
@@ -443,6 +517,7 @@ add_columns (GtkTreeView *treeview)
 														SPEED_COLUMN,
 														NULL);
 	gtk_tree_view_column_set_sort_column_id (column, SPEED_COLUMN);
+	gtk_tree_view_column_set_resizable(column, TRUE); 
 	gtk_tree_view_append_column (treeview, column);
 	
 }
@@ -453,7 +528,9 @@ on_gwget_drag_received (GtkWidget * widget, GdkDragContext * context, int x,
 							guint time, gpointer data)
 {
 	gchar *file;
-	GList *files;
+	gchar *uri;
+	gchar **uris;
+	GList *files = NULL;
 	GwgetData *gwgetdata;
 	gboolean drop_ok;
 	
@@ -463,10 +540,12 @@ on_gwget_drag_received (GtkWidget * widget, GdkDragContext * context, int x,
 	
 	drop_ok = FALSE;
 	if (dnd_type==TARGET_URI_LIST) {
-		GnomeVFSURI *vfs_uri;
-		files = gnome_vfs_uri_list_parse (seldata->data);
-		vfs_uri = files->data;
-		file = gnome_vfs_uri_to_string(vfs_uri,GNOME_VFS_URI_HIDE_NONE);
+		uris = g_uri_list_extract_uris (seldata->data);
+		for (uri=uris[0]; uri != NULL; uri++) {
+			files = g_list_prepend (files, uri);
+		}
+		g_strfreev (uris); 
+		file = files->data;
 		drop_ok = TRUE;
 	} else if (dnd_type==TARGET_NETSCAPE_URL) {
 		file=((gchar *) (seldata->data));
@@ -483,7 +562,7 @@ on_gwget_drag_received (GtkWidget * widget, GdkDragContext * context, int x,
 			new_download(gwgetdata);
 			gwget_data_start_download(gwgetdata);
 		} else {
-			run_dialog(_("Error starting the download"), _("There was an unexpected error starting the download"));
+			run_dialog_error(_("Error starting the download"), _("There was an unexpected error starting the download"));
 		}
 		gtk_drag_finish(context, TRUE, FALSE, time);
 	}
@@ -605,7 +684,7 @@ show_prefered_columns(void)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkitem),FALSE);
 	}
 	checkitem = glade_xml_get_widget(xml_pref,"limit_speed_spin");
-	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(checkitem), gwget_pref.max_speed);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(checkitem), (gdouble)gwget_pref.max_speed);
 
 
 	checkitem = glade_xml_get_widget(xml_pref,"limit_simultaneousdownloads_check");
@@ -615,7 +694,7 @@ show_prefered_columns(void)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkitem),FALSE);
 	}
 	checkitem = glade_xml_get_widget(xml_pref,"limit_simultaneousdownloads_spin");
-	gtk_spin_button_set_digits(GTK_SPIN_BUTTON(checkitem), gwget_pref.max_simultaneousdownloads);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(checkitem), (gdouble)gwget_pref.max_simultaneousdownloads);
 
 }
 
@@ -629,24 +708,24 @@ gwget_remember_window_size_and_position(void)
 	GtkWidget *main_window;
 	GtkAllocation *allocation;
 	gint position_x,position_y;
+	GConfChangeSet *cs;
 
 	/* Remember the size of the window */
 	main_window=glade_xml_get_widget(xml,"main_window");
 	allocation= &(GTK_WIDGET (main_window)->allocation);
-	gconf_client_set_int (gconf_client, 
-			      "/apps/gwget2/default_width",
-			      allocation->width,
-			      NULL);
-	gconf_client_set_int (gconf_client, 
-			      "/apps/gwget2/default_height",
-			      allocation->height,
-			      NULL);
+	cs = gconf_change_set_new ();
+	gconf_change_set_set_int (cs, "/apps/gwget2/default_height", allocation->height);
+	gconf_change_set_set_int (cs, "/apps/gwget2/default_width", allocation->width);
 	
 	/* Remember the position */
 	gtk_window_get_position(GTK_WINDOW(main_window), &position_x, &position_y);
-	gconf_client_set_int (gconf_client,"/apps/gwget2/position_x",position_x,NULL);
-	gconf_client_set_int (gconf_client,"/apps/gwget2/position_y",position_y,NULL);
+	gconf_change_set_set_int (cs, "/apps/gwget2/position_x", position_x);
+	gconf_change_set_set_int (cs, "/apps/gwget2/position_y", position_y);
 	
+	gconf_client_commit_change_set (gconf_client, cs, TRUE, NULL);
+
+	gconf_change_set_unref (cs);
+
 	return FALSE;
 }
 
@@ -661,6 +740,7 @@ gwget_remember_downloads(void)
 	GtkTreeIter iter;
 	gint i,length;
 	gboolean running = FALSE;
+	gchar down_size[2048];
 	
 	/* calculate the number of items in the treeview */
 	length=gtk_tree_model_iter_n_children(GTK_TREE_MODEL(model),NULL);
@@ -693,8 +773,10 @@ gwget_remember_downloads(void)
 		key=g_strdup_printf("/apps/gwget2/downloads_data/%d/state",i);
 		gconf_client_set_int(gconf_client,key,gwgetdata->state,NULL);
 
-		key=g_strdup_printf ("/apps/gwget2/downloads_data/%d/total_size",i);
-		gconf_client_set_int (gconf_client,key,gwgetdata->total_size,NULL);		
+		key=g_strdup_printf ("/apps/gwget2/downloads_data/%d/file_size",i);
+		
+		g_snprintf(down_size,2047, "%" G_GINT64_FORMAT,gwgetdata->total_size);
+		gconf_client_set_string (gconf_client,key,down_size,NULL);		
 		
 		if (gwgetdata->log_tag != -1) 
 		{
@@ -713,12 +795,11 @@ gwget_quit(void)
 	gboolean running;
 	gint response;
 	
-	gwget_remember_window_size_and_position();
 	running = gwget_remember_downloads();
 	
 	if (running) {
-		response = run_dialog(_("Cancel current downloads?"),
-				      _("There is at least one active download left. Really cancel all running transfers?"));
+		response = run_dialog(_("Stop current downloads?"),
+				      _("There is at least one active download left. Really cancel all running transfers?"), _("No"), _("Yes"));
 		if (response == GTK_RESPONSE_OK) {
 			stop_all_downloads();
 			gtk_main_quit();
