@@ -15,6 +15,12 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+ 
+/*
+ * For the log parsing to work, the wget process must be started with
+ * the "C" locale, e.g., by setting the environment variable LC_ALL=C
+ * programmatically prior to invoking wget.
+ */
 
 #define _FILE_OFFSET_BITS 64
 
@@ -25,6 +31,7 @@
 #include <unistd.h>
 #include <signal.h>
 #include <fcntl.h>
+#include <ctype.h>
 #include "gwget_data.h"
 #include "wget-log.h"
 #include "main_window.h"
@@ -58,86 +65,168 @@ show_error (GwgetData *gwgetdata, gchar *error_msg)
     gwget_data_set_state(gwgetdata,DL_ERROR);
 	gwgetdata->error=TRUE;
 	gwgetdata->error_msg=g_strdup(error_msg);
-    
 }
 
+/* Get session start time and session file start size */
+static void
+set_start_size_and_time(GwgetData *gwgetdata)
+{
+	struct stat file_stat;
+	if (stat (gwgetdata->local_filename, &file_stat) != -1) {
+    	gwgetdata->session_start_time = file_stat.st_ctime;
+    	gwgetdata->session_start_size = file_stat.st_size;
+	} else {
+    	gwgetdata->session_start_time = 0;
+    	gwgetdata->session_start_size = 0;
+	}
+}
+
+/* 
+ * If gwgetdata->filename does not match the filesystem filename,
+ * bad things can happen. We intecept the line that prints the 
+ * filesystem filename and set gwgetdata->filename
+ */
+static void
+update_filename(GwgetData *gwgetdata)
+{
+	char *sName = gwgetdata->line;
+	int iL = strlen(sName);
+	sName[iL-1] = 0; // Chop the final ' 
+
+	/*
+	 * Now sName contains the whole pathname. No filename can
+	 * contain '/' so the following search for the last component
+	 * is sane
+	 */
+	sName += iL - 2;
+	while (*sName != '/' && sName != gwgetdata->line)
+		sName--;
+	if (*sName == '/')
+		sName++;
+
+	gwget_data_set_filename(gwgetdata,sName);
+	set_start_size_and_time(gwgetdata);
+	gwget_data_update_statistics(gwgetdata);
+	gwget_remember_downloads();
+}
 
 static int
 wget_log_process_line (GwgetData *gwgetdata)
 {
 	gchar *p;
-	struct stat file_stat;
+	gint dots = 0;
 
-	if (gwgetdata->line == NULL)
+	if ((gwgetdata->line == NULL) || (strlen (gwgetdata->line) == 0))
 		return 0;
-
+		
 	switch (gwgetdata->state) {
 	case DL_NOT_CONNECTED:
-		/* 
-		 * If gwgetdata->filename does not match the filesystem filename,
-		 * bad things can happen. We intecept the line that prints the 
-		 * filesystem filename and set gwgetdata->filename
-		 */
-		if (strstr(gwgetdata->line,"           => `")) {
-			char *sName = gwgetdata->line;
-			int iL = strlen(sName);
-			sName[iL-1] = 0; // Chop the final ' 
+		if (strstr(gwgetdata->line,"           => `")) {		/* wget <1.11 */
+		    update_filename(gwgetdata);
+		    break;
+    	}
+   
+		/* First check to see if we connected to the host correctly. */
 
-			/*
-			 * Now sName contains the whole pathname. No filename can
-			 * contain '/' so the following search for the last component
-			 * is sane
-			 */
-			sName += iL - 2;
-			while (*sName != '/' && sName != gwgetdata->line)
-				sName--;
-			if (*sName == '/')
-				sName++;
-
-			//g_print("NAME: %s\n",sName);
-			gwget_data_set_filename(gwgetdata,sName);
-			gwget_data_update_statistics(gwgetdata);
-			gwget_remember_downloads();
-			break;
-		}
-		  
-		/* First check to see if connected to the host correctly */
 		/* Wget 1.8.1 says "connected." rather than "connected!" */
 		if (strstr (gwgetdata->line, "connected!") != NULL ||
 			strstr (gwgetdata->line, "connected.") != NULL) {
 				gwget_data_set_state (gwgetdata, DL_CONNECTED);
 				break;
 		}
-		/* Wget 1.8.1 adds an explicit "Resolving" status msg */
-		if (strstr (gwgetdata->line, "Resolving") != NULL &&
-            strstr (gwgetdata->line, "Host not found.") == NULL)
-                break;
-
-		/* We are not connected to the host so we must find the problem */
+		
+		/* Second filter out other non-error messages that can precede "connected" */
+		
 		if (strncmp (gwgetdata->line, "--", 2) == 0 ||
-			strncmp (gwgetdata->line, "  ", 2) == 0 ||
-			strncmp (gwgetdata->line, "Connecting to ", 14) == 0 ||
-			strlen(gwgetdata->line) ==0)
+			strncmp (gwgetdata->line, "  ", 2) == 0)
 			break;
-		else if (strncmp (gwgetdata->line, "socket: ", 8) == 0)
+
+		if ((strncmp (gwgetdata->line, "Connecting to ", 14) == 0) &&
+			(strstr (gwgetdata->line, "failed: ") == NULL))
+			break;
+
+		/* Wget 1.8.1 adds an explicit "Resolving" status msg */
+		if ((strstr (gwgetdata->line, "Resolving") != NULL) &&
+            (strstr (gwgetdata->line, "Host not found.") == NULL) &&
+			(strstr (gwgetdata->line, "Name or service not known") == NULL))
+                break;
+			
+		/* Wget, under certain circumstances, returns a list of resolved IP addresses
+		 *	before attempting to connect, which can be ignored.
+		 */
+		
+		/* Test for ipv4 address */
+		dots = 0;
+		for (p = gwgetdata->line; p[0]; p ++) {
+			if (isdigit (p[0])) continue;
+			else if (p[0] == '.') dots ++;
+			else if ((p[0] == '\n') || (p[0] == ',')) break;
+			else break;
+		}
+		if ((! p[0]) || (p[0] == '\n') || (p[0] == ',')) {
+			if (dots == 3) break;
+		}
+
+		/* Test for ipv6 address */
+		dots = 0;
+		for (p = gwgetdata->line; p[0]; p ++) {
+			if (isxdigit (p[0])) continue;
+			else if (p[0] == ':') dots ++;
+			else if ((p[0] == '\n') || (p[0] == ',')) break;
+			else break;
+		}
+		if ((! p[0]) || (p[0] == '\n') || (p[0] == ',')) {
+			if (dots > 1) break;
+		}
+
+		/* Failing the above checks, we _assume_ we have an error. The wget process will
+		 * be killed after we create an error message.
+		 */
+
+		if (strncmp (gwgetdata->line, "socket: ", 8) == 0)
 			show_error (gwgetdata, _ ("Socket error"));
+			
 	    else if (strncmp (gwgetdata->line, "Connection to ", 14) == 0)
-			show_error (gwgetdata, _ ("Connection refused\n"));
+			show_error (gwgetdata, _ ("Connection refused"));
+			
 	    else if (strstr (gwgetdata->line, "No route to host") != NULL)
 			show_error (gwgetdata, _ ("No route to host"));
+			
 	    else if (strncmp (gwgetdata->line, "connect: ", 9) == 0)
-				show_error (gwgetdata, _ ("Connection refused when downloading URL:\n"));
+			show_error (gwgetdata, _ ("Connection refused when downloading URL:"));
+			
 	    else if (strstr (gwgetdata->line, "Host not found.") != NULL)
-			show_error (gwgetdata, _ ("Host not found\n"));
+			show_error (gwgetdata, _ ("Host not found"));
+			
+		else if (strstr (gwgetdata->line, "Name or service not known") != NULL)
+			show_error (gwgetdata, _ ("Name or service not known"));
+			
 	    else if (strstr (gwgetdata->line, "unsupported protocol") != NULL)
-		show_error (gwgetdata, _ ("Unsupported protocol - you need wget >= 1.7 "
-			       "for https:\n"));
+			show_error (gwgetdata, _ ("Unsupported protocol"));
+			
 		else if (strstr (gwgetdata->line, "Refusing to truncate existing") != NULL)
 			show_error(gwgetdata, _("Refusing to truncate existing file"));
-	    else {
-			show_error (gwgetdata, _ ("Unknown error"));
 			
+		else if (strstr (gwgetdata->line, "unable to resolve"))
+				show_error (gwgetdata, _ ("Unable to resolve host address"));
+			
+		else if ((p = strstr (gwgetdata->line, "failed: ")) != NULL) {
+			/* 
+			 * This is somewhat generic looking, but is informative
+			 */
+			show_error (gwgetdata, p + 8);
 		}
+			
+		else {
+			/*
+			 * All other possible output may as well be reported, since we treat it
+			 * as an error. We tag the message as unknown to make it more meaningful.
+			 */
+			p = g_strconcat (_ ("Unknown error "), gwgetdata->line, NULL);
+			show_error (gwgetdata, p);
+			g_free (p);
+		}
+		
 		kill (gwgetdata->wget_pid, SIGKILL);
 		return 1;
 		break;
@@ -152,6 +241,7 @@ wget_log_process_line (GwgetData *gwgetdata)
 
 		/* File not found for HTTP or Proxy */
 		if (strstr (gwgetdata->line, "ERROR") != NULL) {
+			gwget_data_set_state (gwgetdata, DL_ERROR);
 			show_error (gwgetdata, _ ("File not found"));
 			break;
 	    }
@@ -185,14 +275,7 @@ wget_log_process_line (GwgetData *gwgetdata)
 			}
 			gwget_data_set_state (gwgetdata, DL_RETRIEVING);
 			set_icon_newdownload();
-			/* Get session start time and session file start size */
-			if (stat (gwgetdata->local_filename, &file_stat) != -1) {
-		    	gwgetdata->session_start_time = file_stat.st_ctime;
-		    	gwgetdata->session_start_size = file_stat.st_size;
-			} else {
-		    	gwgetdata->session_start_time = 0;
-		    	gwgetdata->session_start_size = 0;
-			}
+			set_start_size_and_time(gwgetdata);
 			gwgetdata->session_elapsed = 0;
 	    } else {
 				/* We didn't get a length so, probably it's unspecified size
@@ -203,19 +286,18 @@ wget_log_process_line (GwgetData *gwgetdata)
 					/* Unspecified size, so set total_size to 0 */
 					gwget_data_set_total_size (gwgetdata, 0);
 					gwget_data_set_state (gwgetdata, DL_RETRIEVING);
-					if (stat (gwgetdata->local_filename, &file_stat) != -1) {
-						gwgetdata->session_start_time = file_stat.st_ctime;
-						gwgetdata->session_start_size = file_stat.st_size;
-					} else {
-						gwgetdata->session_start_time = 0;
-						gwgetdata->session_start_size = 0;
-					}
+					set_start_size_and_time(gwgetdata);
 				}
 				gwgetdata->session_elapsed = 0;
         }
 	    break;
 
 		case DL_RETRIEVING:
+			if (strstr(gwgetdata->line,"Saving to: `")) {		/* wget >=1.11 */
+			    update_filename(gwgetdata);
+			    break;
+	    	}
+
 			if (strncmp (gwgetdata->line, "Cannot write to ", 15) == 0) {
 			show_error (gwgetdata,
 					    _ ("Can't write to target directory"));
@@ -230,14 +312,7 @@ wget_log_process_line (GwgetData *gwgetdata)
 				if (p != NULL) {
 					p += 8;
 					gwget_data_set_total_size (gwgetdata,convert_wget_size (p));
-					/* Get session start time and session file start size */
-					if (stat (gwgetdata->local_filename, &file_stat) != -1) {
-						gwgetdata->session_start_time = file_stat.st_ctime;
-						gwgetdata->session_start_size = file_stat.st_size;
-					} else {
-						gwgetdata->session_start_time = 0;
-						gwgetdata->session_start_size = 0;
-					}
+					set_start_size_and_time(gwgetdata);
 					gwgetdata->session_elapsed = 0;
 				} else {
 		                	/* We didn't get a length so, probably it's unspecified size
@@ -247,13 +322,7 @@ wget_log_process_line (GwgetData *gwgetdata)
 					if (p != NULL) {
 						/* Unspecified size, so set total_size to 0 */
 						gwget_data_set_total_size (gwgetdata, 0);
-						if (stat (gwgetdata->local_filename, &file_stat) != -1) {
-							gwgetdata->session_start_time = file_stat.st_ctime;
-							gwgetdata->session_start_size = file_stat.st_size;
-						} else {
-							gwgetdata->session_start_time = 0;
-							gwgetdata->session_start_size = 0;
-						}
+						set_start_size_and_time(gwgetdata);
 					}
 					gwgetdata->session_elapsed = 0;
 				}
@@ -296,82 +365,73 @@ wget_log_process_line (GwgetData *gwgetdata)
 	return 0;
 }
 
-static void
+static gboolean
 wget_log_read_log_line(GwgetData *gwgetdata) {
 	char c;
-	int iRes;
-	int iBlockCount;
-	gchar *buffer;
-	int iWritePos;
+	int res;
 
-	g_free(gwgetdata->line);
-	gwgetdata->line = NULL;
-
-	iRes = read(gwgetdata->log_fd,&c,1);
-
-	if (iRes < 1) {
+	res = read (gwgetdata->log_fd, &c, 1);
+	if (res < 1) {
 		/*
 		 * No input available
 		 */
-		gwgetdata->line = NULL;
-		return;
+		return FALSE;
 	}
 
-	iBlockCount = 1;
-	buffer = g_malloc(sizeof(gchar)*(iBlockCount*BLOCK_SIZE));
-	iWritePos = 0;
-		
-	buffer[iWritePos++] = c;
-	while (c != '\n') {
-		iRes = read(gwgetdata->log_fd,&c,1);
-		if (iRes < 1) {
-			/*
-			 * There is currently no more data to read. Return what we have.
-			 */
-			buffer[iWritePos++] = 0;
-			break;
-		}
-		buffer[iWritePos++] = c;
+	if (! gwgetdata->line) {
+		gwgetdata->line = g_malloc (sizeof (gchar) * BLOCK_SIZE);
+		gwgetdata->line_pos = 0;
+		gwgetdata->line_blocks = 1;
+	}
 
-		if (iWritePos == iBlockCount*BLOCK_SIZE && c != '\n') {
-			/*
-			 * The buffer is full , expanding
-			 */
-			iBlockCount++;
-			buffer = g_realloc(buffer,
-					   sizeof(gchar)*(iBlockCount*BLOCK_SIZE));
-		}
-	} 
-	buffer[iWritePos-1] = 0;
-	//g_print("LOG: %s\n",buffer);
+	gwgetdata->line [gwgetdata->line_pos ++] = c;
 	
-	gwgetdata->line = buffer;
+	while (c != '\n') {
+		res = read (gwgetdata->log_fd, &c, 1);
+		if (res < 1) {
+			/*
+			 * There is currently no more data to read, so return FALSE - but 
+			 * the line can still be completed later where it left off.
+			 */ 
+			return FALSE;
+		}
+
+		gwgetdata->line [gwgetdata->line_pos ++] = c;
+		if ((gwgetdata->line_pos == gwgetdata->line_blocks * BLOCK_SIZE) && (c != '\n')) {
+			/*
+			 * The buffer is full, expanding
+			 */
+			gwgetdata->line_blocks ++;
+			gwgetdata->line = g_realloc (gwgetdata->line,
+					   sizeof (gchar) * gwgetdata->line_blocks * BLOCK_SIZE);
+		}
+	}
+	
+	gwgetdata->line [gwgetdata->line_pos -1] = 0;
+
+	/* 
+	 * We can just reuse gwgetdata->line by setting the line_pos to zero.
+	 */
+	gwgetdata->line_pos = 0;
+	
+	/*
+	 * Result TRUE means gwgetdata->line contains a complete line.
+	 */ 
+	return TRUE;
 }
 
 void
 wget_drain_remaining_log(GwgetData *gwgetdata) 
 {
-	wget_log_read_log_line(gwgetdata);
-	while (gwgetdata->line != NULL) {
-		wget_log_process_line(gwgetdata);
-		wget_log_read_log_line(gwgetdata);
-	}
-	gwget_data_update_statistics(gwgetdata);
+	wget_log_process (gwgetdata);
 }
 
 void
 wget_log_process (GwgetData *gwgetdata)
 {
-	/*
-	 * Read and process two lines of log
-	 */
-	wget_log_read_log_line(gwgetdata);
-	wget_log_process_line(gwgetdata);
+	while (wget_log_read_log_line (gwgetdata))
+		wget_log_process_line (gwgetdata);
 	
-	wget_log_read_log_line(gwgetdata);
-	wget_log_process_line(gwgetdata);
-
-	if (gwgetdata->state == DL_RETRIEVING)
-		gwget_data_update_statistics(gwgetdata);
+	 if (gwgetdata->state == DL_RETRIEVING)
+		gwget_data_update_statistics (gwgetdata);
 }
-
